#!/bin/bash

#
### number of macroparticles used in the simulations. 
num_MP=`rpnl 1e3`
##
# 0, necessary values
#   (1), total beam energy, unit is [MeV]
Energy_Tot_in_MeV=`rpnl 6000`
#
#   (2), the initial vertical offset of the beam centroid, unit is [m]
DY_ini=`rpnl 1e\-6`
#
#   (3), the vertical chromaticity
chrom_y=`rpnl 0`
#
#   (4), the initial horizontal beam emittance, unit is [m]
emit_x_ini=`rpnl 59\.4e\-12`
#
#   (5), the initial vertical beam emittance, unit is [m]
emit_y_ini=`rpnl 5\.94e\-12`
#
#   (6), the initial beam energy spread
sigma_dp_ini=`rpnl 8\.0e\-4`
#
#   (7), the initial bunch length, unit is [m]
sigma_z_ini=`rpnl 6e\-3`

# 1, Calculate the Optics parameters
elegant Basic.ele 2>&1 | tee Basic.log

./ILMatrixSetup
./SREffectsSetup

# 2, set up the RF parameters
#   (1), no HC
# Average energy loss per turn, unit is [MeV]
U0=`sdds2stream Basic.twi -par=U0`
#
# Peak RF voltage of the primary cavity, unit is [eV]
V_RF=`sdds2stream Basic.rf -par=Voltage`
#
# Synchronous phase of the primary cavity
phi_RF=`rpnl 180 $U0 1e6 \* $V_RF / asin 180 \* pi / -`
#
# Peak RF voltage of the harmonic acvity, unit is [eV]
V_HC=0.0
#
# Synchronous phase of the harmonic cavity
phi_HC=0.0

# 3, impedance information preparation
# define the number of turns used in the simulation
nturns=4000
#
# define the number of f_0 in the simulations.
fmax_over_f0_zt=15
#
# number of bins used in the simulations. This value is better to be (2^n+1), such as 1025, 2049, 4097, 8193, 16385, 32769, 65537.
nbins_zt=16385
#

# 4, define the folder which is used to save the simulation results.
results=Results_${fmax_over_f0_zt}f0_${nbins_zt}bins_${num_MP}MP
if [ ! -d "$results" ];
then
    mkdir $results
fi

# 5, main loop, scanning the single-bunch charge
# first define the single-bunch charge in the unit of [nC]
n_charge_del=`rpnl 865.0`

freq_scan=(
700.800e6
700.840e6
700.880e6
700.920e6
700.960e6
)

# total number of scan
Nscan=${#freq_scan[@]}

for ((i=0;i<$Nscan;i++))
do
    freq_scan_value=${freq_scan[$i]}
    ncharge=${n_charge_del}
    # define the rootname of this simulation;
    rootname=${results}/freq_${freq_scan_value}
    if [ ! -d "$rootname" ];
    then
        mkdir $rootname
    fi
    #
    # single-bunch charge, unit is [C]
    charge=`rpnl $ncharge 1e-9 \*`
    #
    # define the names of the .sdds files and the corresponding paths.
    tempfile=${rootname}/beam_temp.sdds
    TMCI_infile=${rootname}/beam_TMCI_in.sdds
    #
    #### <1> the Gaussian bunch is generated by the random number generator (not the 'halton_sequence' in this case)
    elegant PrepareBunch.ele -macro=nPerBunch=${num_MP},beamfile=${TMCI_infile} 2>&1 | tee PrepareBunch_${ncharge}.log
    sigma_t=`rpnl 20e-12`
    #
    # <2> the 'equilibrium' final distribution given by the Haissinski solver
    #
    # the characteristic frequency of the Gaussian bunch, unit is [Hz]
    f0=`rpnl 1 $sigma_t / 2 / pi /`
    #
    # the maximum frequency in the impedance datafile, which is determined by the f0 and number of f0
    fmax_zt=`rpnl $f0 $fmax_over_f0_zt \*`
    #
    # define files and data for the simulations
    wp00=${rootname}/wp00.sdds
    wc00=${rootname}/wc00.sdds
    h00=${rootname}/h00.sdds
    twi=${rootname}/mwitrack.twi
    #
    # create the machine file of the 'mpiexec' command
    num_threads=88
    hostname > mf_temp
    sed "s/ihep.ac.cn/ihep.ac.cn:${num_threads}/g" mf_temp > mf
    rm mf_temp
    # run the simulations
    mpd &

    mpiexec -machinefile mf -np ${num_threads} Pelegant Collective_Track_Template.ele -macro=Energy_in_MeV=${Energy_Tot_in_MeV},V_RF=${V_RF},Phi_RF=${phi_RF},turns=${nturns},beam=${TMCI_infile},wp00=${wp00},wc00=${wc00},twi=${twi},charge=${charge},DY_INI=${DY_ini},chrom_y=${chrom_y},frequencyMode=${freq_scan_value} 2>&1 | tee ${rootname}/log.txt 

    mv *.rfmm ${rootname}/ 
    mv *.h00 ${rootname}/
done

